@@ -68,10 +68,11 @@
 #undef STATIC_GSSAPIV2
 #undef STATIC_KERBEROS4
 #undef STATIC_LOGIN
+#undef STATIC_MYSQL
 #undef STATIC_OTP
 #undef STATIC_PLAIN
-#undef STATIC_SRP
 #undef STATIC_SASLDB
+#undef STATIC_SRP
 
 /* This is where plugins will live at runtime */
 #undef PLUGINDIR
@@ -149,9 +150,6 @@
 /* do we have a preferred mechanism, or should we just pick the highest ssf? */
 #undef PREFER_MECH
 
-/* do we have a wierd location of db.h? */
-#undef HAVE_DB3_DB_H
-
 /* define if your compile has __attribute__ */
 #undef HAVE___ATTRIBUTE__
 
@@ -183,6 +181,9 @@
 #undef HAVE_SOCKLEN_T
 #undef HAVE_SOCKADDR_SA_LEN
 
+/* do we use doors for IPC? */
+#undef USE_DOORS
+
 @BOTTOM@
 
 /* Create a struct iovec if we need one */
