@@ -1,7 +1,7 @@
 /* SASL server API implementation
  * Rob Siemborski
  * Tim Martin
- * $Id: checkpw.c,v 1.49 2002/03/07 19:14:04 ken3 Exp $
+ * $Id: checkpw.c,v 1.56 2002/05/22 16:20:28 leg Exp $
  */
 /* 
  * Copyright (c) 2001 Carnegie Mellon University.  All rights reserved.
@@ -57,6 +57,11 @@
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
+#include <fcntl.h>
+#ifdef USE_DOORS
+#include <sys/mman.h>
+#include <door.h>
+#endif
 
 #include <stdlib.h>
 
@@ -138,7 +143,7 @@ static int auxprop_verify_password(sasl_conn_t *conn,
     int result = SASL_OK;
     sasl_server_conn_t *sconn = (sasl_server_conn_t *)conn;
     const char *password_request[] = { SASL_AUX_PASSWORD,
-				       "cmusaslsecretPLAIN",
+				       "*cmusaslsecretPLAIN",
 				       NULL };
     struct propval auxprop_values[3];
     
@@ -156,13 +161,10 @@ static int auxprop_verify_password(sasl_conn_t *conn,
     if(result != SASL_OK) return result;
 
     result = _sasl_canon_user(conn, userstr, 0,
-			      SASL_CU_AUTHID, &(conn->oparams));
+			      SASL_CU_AUTHID | SASL_CU_AUTHZID,
+			      &(conn->oparams));
     if(result != SASL_OK) return result;
     
-    result = _sasl_canon_user(conn, userstr, 0,
-			      SASL_CU_AUTHZID, &(conn->oparams));
-    if(result != SASL_OK) return result;
-
     result = prop_getnames(sconn->sparams->propctx, password_request,
 			   auxprop_values);
     if(result < 0)
@@ -247,6 +249,7 @@ int _sasl_auxprop_verify_apop(sasl_conn_t *conn,
        PARAMERROR(conn)
 
     /* We've done the auxprop lookup already (in our caller) */
+    /* sadly, APOP has no provision for storing secrets */
     ret = prop_getnames(sconn->sparams->propctx, password_request,
 			auxprop_values);
     if(ret < 0) {
@@ -361,10 +364,8 @@ static int pwcheck_verify_password(sasl_conn_t *conn,
     int r;
     struct iovec iov[10];
     static char response[1024];
-    int start, n;
+    unsigned start, n;
     char pwpath[1024];
-    sasl_getopt_t *getopt;
-    void *context;
 
     if (strlen(PWCHECKDIR)+8+1 > sizeof(pwpath)) return SASL_FAIL;
 
@@ -441,20 +442,23 @@ static int retry_read(int fd, void *buf, unsigned nbyte)
 
 /* saslauthd-authenticated login */
 static int saslauthd_verify_password(sasl_conn_t *conn,
-				   const char *userid, 
-				   const char *passwd,
-				   const char *service,
-				   const char *user_realm)
+				     const char *userid, 
+				     const char *passwd,
+				     const char *service,
+				     const char *user_realm)
 {
-    static char response[1024];
+    char response[1024];
+    char query[8192];
+    char *query_end = query;
     int s;
     struct sockaddr_un srvaddr;
-    int r;
-    unsigned short count;
     sasl_getopt_t *getopt;
     void *context;
     char pwpath[sizeof(srvaddr.sun_path)];
     const char *p = NULL;
+#ifdef USE_DOORS
+    door_arg_t arg;
+#endif
 
     /* check to see if the user configured a rundir */
     if (_sasl_getcallback(conn, SASL_CB_GETOPT, &getopt, &context) == SASL_OK) {
@@ -470,20 +474,6 @@ static int saslauthd_verify_password(sasl_conn_t *conn,
 	strcat(pwpath, "/mux");
     }
 
-    s = socket(AF_UNIX, SOCK_STREAM, 0);
-    if (s == -1)
-	return errno;
-
-    memset((char *)&srvaddr, 0, sizeof(srvaddr));
-    srvaddr.sun_family = AF_UNIX;
-    strncpy(srvaddr.sun_path, pwpath, sizeof(srvaddr.sun_path));
-
-    r = connect(s, (struct sockaddr *) &srvaddr, sizeof(srvaddr));
-    if (r == -1) {
-	sasl_seterror(conn, 0, "cannot connect to saslauthd server");
-	return SASL_FAIL;
-    }
-
     /*
      * build request of the form:
      *
@@ -491,66 +481,134 @@ static int saslauthd_verify_password(sasl_conn_t *conn,
      */
     {
  	unsigned short u_len, p_len, s_len, r_len;
+ 
+ 	u_len = (strlen(userid));
+ 	p_len = (strlen(passwd));
+	s_len = (strlen(service));
+	r_len = ((user_realm ? strlen(user_realm) : 0));
+
+	if (u_len + p_len + s_len + r_len + 30 > (unsigned short) sizeof(query)) {
+	    /* request just too damn big */
+            sasl_seterror(conn, 0, "saslauthd request too large");
+	    return SASL_FAIL;
+	}
+
+	u_len = htons(u_len);
+	p_len = htons(p_len);
+	s_len = htons(s_len);
+	r_len = htons(r_len);
+
+	memcpy(query_end, &u_len, sizeof(unsigned short));
+	query_end += sizeof(unsigned short);
+	while (*userid) *query_end++ = *userid++;
+
+	memcpy(query_end, &p_len, sizeof(unsigned short));
+	query_end += sizeof(unsigned short);
+	while (*passwd) *query_end++ = *passwd++;
+
+	memcpy(query_end, &s_len, sizeof(unsigned short));
+	query_end += sizeof(unsigned short);
+	while (*service) *query_end++ = *service++;
+
+	memcpy(query_end, &r_len, sizeof(unsigned short));
+	query_end += sizeof(unsigned short);
+	if (user_realm) while (*user_realm) *query_end++ = *user_realm++;
+    }
+
+#ifdef USE_DOORS
+    s = open(pwpath, O_RDONLY);
+    if (s < 0) {
+	sasl_seterror(conn, 0, "cannot open door to saslauthd server: %m");
+	return SASL_FAIL;
+    }
+
+    arg.data_ptr = query;
+    arg.data_size = query_end - query;
+    arg.desc_ptr = NULL;
+    arg.desc_num = 0;
+    arg.rbuf = response;
+    arg.rsize = sizeof(response);
+
+    door_call(s, &arg);
+
+    if (arg.data_ptr != response || arg.data_size >= sizeof(response)) {
+	/* oh damn, we got back a really long response */
+	munmap(arg.rbuf, arg.rsize);
+	sasl_seterror(conn, 0, "saslauthd sent an overly long response");
+	return SASL_FAIL;
+    }
+    response[arg.data_size] = '\0';
+
+    close(s);
+#else
+    /* unix sockets */
+
+    s = socket(AF_UNIX, SOCK_STREAM, 0);
+    if (s == -1) {
+	sasl_seterror(conn, 0, "cannot create socket for saslauthd: %m");
+	return SASL_FAIL;
+    }
+
+    memset((char *)&srvaddr, 0, sizeof(srvaddr));
+    srvaddr.sun_family = AF_UNIX;
+    strncpy(srvaddr.sun_path, pwpath, sizeof(srvaddr.sun_path));
+
+    {
+	int r = connect(s, (struct sockaddr *) &srvaddr, sizeof(srvaddr));
+	if (r == -1) {
+	    sasl_seterror(conn, 0, "cannot connect to saslauthd server: %m");
+	    return SASL_FAIL;
+	}
+    }
+
+    {
  	struct iovec iov[8];
  
- 	u_len = htons(strlen(userid));
- 	p_len = htons(strlen(passwd));
-	s_len = htons(strlen(service));
-	r_len = htons((user_realm ? strlen(user_realm) : 0));
+	iov[0].iov_len = query_end - query;
+	iov[0].iov_base = query;
 
-	iov[0].iov_base = &u_len;
- 	iov[0].iov_len = sizeof(u_len);
-        iov[1].iov_base = (void*) userid;
-	iov[1].iov_len = strlen(userid);
-	iov[2].iov_base = &p_len;
-	iov[2].iov_len = sizeof(p_len);
-	iov[3].iov_base = (void*) passwd;
-	iov[3].iov_len = strlen(passwd);
-	iov[4].iov_base = &s_len;
-	iov[4].iov_len = sizeof(s_len);
-	iov[5].iov_base = (void*) service;
-	iov[5].iov_len = strlen(service);
-	iov[6].iov_base = &r_len;
-        iov[6].iov_len = sizeof(r_len);
-	iov[7].iov_base = user_realm ? (void*) user_realm : "";
-	iov[7].iov_len = user_realm ? strlen(user_realm) : 0;
-
-	if (retry_writev(s, iov, 8) == -1) {
+	if (retry_writev(s, iov, 1) == -1) {
             sasl_seterror(conn, 0, "write failed");
   	    return SASL_FAIL;
   	}
     }
-  
-    /*
-     * read response of the form:
-     *
-     * count result
-     */
-    if (retry_read(s, &count, sizeof(count)) < (int) sizeof(count)) {
-        sasl_seterror(conn, 0, "size read failed");
-	return SASL_FAIL;
+
+    {
+	unsigned short count = 0;
+
+	/*
+	 * read response of the form:
+	 *
+	 * count result
+	 */
+	if (retry_read(s, &count, sizeof(count)) < (int) sizeof(count)) {
+	    sasl_seterror(conn, 0, "size read failed");
+	    return SASL_FAIL;
+	}
+	
+	count = ntohs(count);
+	if (count < 2) { /* MUST have at least "OK" or "NO" */
+	    close(s);
+	    sasl_seterror(conn, 0, "bad response from saslauthd");
+	    return SASL_FAIL;
+	}
+	
+	count = (int)sizeof(response) < count ? sizeof(response) : count;
+	if (retry_read(s, response, count) < count) {
+	    close(s);
+	    sasl_seterror(conn, 0, "read failed");
+	    return SASL_FAIL;
+	}
+	response[count] = '\0';
     }
-  
-    count = ntohs(count);
-    if (count < 2) { /* MUST have at least "OK" or "NO" */
-	close(s);
-        sasl_seterror(conn, 0, "bad response from saslauthd");
-	return SASL_FAIL;
-    }
-  
-    count = (int)sizeof(response) < count ? sizeof(response) : count;
-    if (retry_read(s, response, count) < count) {
-	close(s);
-        sasl_seterror(conn, 0, "read failed");
-	return SASL_FAIL;
-    }
-  
+
     close(s);
+#endif /* USE_DOORS */
   
-    if (!strncmp(response, "OK", 2))
-      return SASL_OK;
+    if (!strncmp(response, "OK", 2)) {
+	return SASL_OK;
+    }
   
-    response[count] = '\0';
     sasl_seterror(conn, SASL_NOLOG, "authentication failed");
     return SASL_BADAUTH;
 }
