@@ -1,7 +1,7 @@
 /* SASL server API implementation
  * Rob Siemborski
  * Tim Martin
- * $Id: client.c,v 1.46 2002/02/21 21:28:16 rjs3 Exp $
+ * $Id: client.c,v 1.53 2002/07/02 19:01:40 rjs3 Exp $
  */
 /* 
  * Copyright (c) 2001 Carnegie Mellon University.  All rights reserved.
@@ -205,7 +205,7 @@ int sasl_client_init(const sasl_callback_t *callbacks)
   if (ret!=SASL_OK)
     return ret;
 
-  sasl_client_add_plugin("EXTERNAL", &external_client_init);
+  sasl_client_add_plugin("EXTERNAL", &external_client_plug_init);
 
   ret = _sasl_common_init();
 
@@ -482,6 +482,30 @@ int sasl_client_start(sasl_conn_t *conn,
 	    }
 #endif
 
+	    /* compare security flags, only take new mechanism if it has
+	     * more security flags than the previous one.
+	     *
+	     * From the mechanisms we ship with, this yields the order:
+	     *
+	     * SRP
+	     * GSSAPI + KERBEROS_V4
+	     * DIGEST + OTP
+	     * CRAM + EXTERNAL
+	     * PLAIN + LOGIN + ANONYMOUS
+	     *
+	     * This might be improved on by comparing the numeric value of
+	     * the bitwise-or'd security flags, which splits DIGEST/OTP,
+	     * CRAM/EXTERNAL, and PLAIN/LOGIN from ANONYMOUS, but then we
+	     * are depending on the numeric values of the flags (which may
+	     * change, and their ordering could be considered dumb luck.
+	     */
+
+	    if (bestm &&
+		((m->plug->security_flags ^ bestm->plug->security_flags) &
+		 bestm->plug->security_flags)) {
+		break;
+	    }
+
 	    if (mech) {
 		*mech = m->plug->mech_name;
 	    }
@@ -498,14 +522,21 @@ int sasl_client_start(sasl_conn_t *conn,
     }
 
     /* make cparams */
-    c_conn->cparams->serverFQDN = c_conn->serverFQDN; 
     c_conn->cparams->service = conn->service;
+    c_conn->cparams->servicelen = strlen(conn->service);
+    
+    c_conn->cparams->serverFQDN = c_conn->serverFQDN; 
+    c_conn->cparams->servicelen = strlen(c_conn->serverFQDN);
+
+    c_conn->cparams->clientFQDN = NULL; /* XXX */
+    c_conn->cparams->clen = 0;
+
     c_conn->cparams->external_ssf = conn->external.ssf;
     c_conn->cparams->props = conn->props;
     c_conn->mech = bestm;
 
     /* init that plugin */
-    result = c_conn->mech->plug->mech_new(NULL,
+    result = c_conn->mech->plug->mech_new(c_conn->mech->plug->glob_context,
 					  c_conn->cparams,
 					  &(conn->context));
     if(result != SASL_OK) goto done;
@@ -560,6 +591,12 @@ int sasl_client_step(sasl_conn_t *conn,
   if ((serverin==NULL) && (serverinlen>0))
       PARAMERROR(conn);
 
+  /* Don't do another step if the plugin told us that we're done */
+  if (conn->oparams.doneflag) {
+      _sasl_log(conn, SASL_LOG_ERR, "attempting client step after doneflag");
+      return SASL_FAIL;
+  }
+
   if(clientout) *clientout = NULL;
   if(clientoutlen) *clientoutlen = 0;
 
@@ -619,7 +656,7 @@ int _sasl_client_listmech(sasl_conn_t *conn,
 			  const char *sep,
 			  const char *suffix,
 			  const char **result,
-			  size_t *plen,
+			  unsigned *plen,
 			  int *pcount)
 {
     cmechanism_t *m=NULL;
