@@ -1,7 +1,7 @@
 /* common.c - Functions that are common to server and clinet
  * Rob Siemborski
  * Tim Martin
- * $Id: common.c,v 1.79 2002/05/17 19:41:46 rjs3 Exp $
+ * $Id: common.c,v 1.81 2002/06/19 18:07:23 rjs3 Exp $
  */
 /* 
  * Copyright (c) 2001 Carnegie Mellon University.  All rights reserved.
@@ -700,17 +700,23 @@ int sasl_setprop(sasl_conn_t *conn, int propnum, const void *value)
 	  if(conn->type == SASL_CONN_CLIENT) {
 	      ((sasl_client_conn_t *)conn)->cparams->ipremoteport
 		  = conn->ipremoteport;
+	      ((sasl_client_conn_t *)conn)->cparams->ipremlen =
+		  strlen(conn->ipremoteport);
 	  } else if (conn->type == SASL_CONN_SERVER) {
 	      ((sasl_server_conn_t *)conn)->sparams->ipremoteport
 		  = conn->ipremoteport;
+	      ((sasl_server_conn_t *)conn)->sparams->ipremlen =
+		  strlen(conn->ipremoteport);
 	  }
       } else {
 	  if(conn->type == SASL_CONN_CLIENT) {
 	      ((sasl_client_conn_t *)conn)->cparams->ipremoteport
 		  = NULL;
+	      ((sasl_client_conn_t *)conn)->cparams->ipremlen = 0;
 	  } else if (conn->type == SASL_CONN_SERVER) {
 	      ((sasl_server_conn_t *)conn)->sparams->ipremoteport
-		  = NULL;
+		  = NULL;	      
+	      ((sasl_server_conn_t *)conn)->sparams->ipremlen = 0;
 	  }
       }
 
@@ -735,17 +741,23 @@ int sasl_setprop(sasl_conn_t *conn, int propnum, const void *value)
 	  if(conn->type == SASL_CONN_CLIENT) {
 	      ((sasl_client_conn_t *)conn)->cparams->iplocalport
 		  = conn->iplocalport;
+	      ((sasl_client_conn_t *)conn)->cparams->iploclen
+		  = strlen(conn->iplocalport);
 	  } else if (conn->type == SASL_CONN_SERVER) {
 	      ((sasl_server_conn_t *)conn)->sparams->iplocalport
 		  = conn->iplocalport;
+	      ((sasl_server_conn_t *)conn)->sparams->iploclen
+		  = strlen(conn->iplocalport);
 	  }
       } else {
 	  if(conn->type == SASL_CONN_CLIENT) {
 	      ((sasl_client_conn_t *)conn)->cparams->iplocalport
 		  = NULL;
+	      ((sasl_client_conn_t *)conn)->cparams->iploclen = 0;
 	  } else if (conn->type == SASL_CONN_SERVER) {
 	      ((sasl_server_conn_t *)conn)->sparams->iplocalport
 		  = NULL;
+	      ((sasl_server_conn_t *)conn)->sparams->iploclen = 0;
 	  }
       }
       break;
@@ -846,7 +858,7 @@ static int _sasl_global_getopt(void *context,
 			       const char *plugin_name,
 			       const char *option,
 			       const char ** result,
-			       size_t *len)
+			       unsigned *len)
 {
   const sasl_global_callbacks_t * global_callbacks;
   const sasl_callback_t *callback;
@@ -1503,7 +1515,7 @@ int _buf_alloc(char **rwbuf, size_t *curlen, size_t newlen)
 
 /* for the mac os x cfm glue: this lets the calling function
    get pointers to the error buffer without having to touch the sasl_conn_t struct */
-void _sasl_get_errorbuf(sasl_conn_t *conn, char ***bufhdl, unsigned **lenhdl)
+void _sasl_get_errorbuf(sasl_conn_t *conn, char ***bufhdl, size_t **lenhdl)
 {
 	*bufhdl = &conn->error_buf;
 	*lenhdl = &conn->error_buf_len;
